// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/utils/util_fsamp_ai.c $
// $Rev: 51943 $
// $Date: 2022-10-25 08:29:04 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	ABS(v)					(((v) < 0) ? (-(v)) : (v))
#define	LIMIT_MAX(v,max)		(((v) > (max)) ? (max) : (v))
#define	LIMIT_MIN(v,min)		(((v) < (min)) ? (min) : (v))
#define	LIMIT_RANGE(v,min,max)	(LIMIT_MIN((LIMIT_MAX((v),(max))),(min)))



/******************************************************************************
*
*	Function:	sdi_fsamp_ai_compute
*
*	Purpose:
*
*		Calculate the best values for the specified sample rate.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	The desired sample rate.
*
*		nrate	Put the resulting Nrate value here.
*
*		ndiv	Put the resulting Ndiv value here.
*
*		rate	Report the sample rate achieved here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sdi_fsamp_ai_compute(
	int			fd,
	int			index,
	int			verbose,
	s32			fsamp,
	s32*		nrate,
	s32*		ndiv,
	double*		rate)
{
	double	delta;
	double	delta_best	= FLT_MAX;
	s32		dev_type;
	int		errs		= 0;
	double	fgen_got;
	s32		fgen_max;
	s32		fgen_min;
	double	fsamp_best	= 0;
	double	fsamp_got;
	s32		fsamp_max;	// Queried value.
	s32		fsamp_min;	// Queried value.
	double	fsamp_want;
	int		init		= 0;	// Have we performed the first run initialization?
	s32		ndiv_best	= 0;
	s32		ndiv_inc;
	s32		ndiv_max;
	s32		ndiv_min;
	s32		nrate_best	= 0;
	s32		nrate_inc;
	s32		nrate_max;
	s32		nrate_min;

	if (verbose)
	{
		gsc_label_index("AI Sample Rate", index);
		gsc_label_long_comma(fsamp);
		printf(" S/S");
	}

	errs	+= sdi_query(fd, -1, 0, SDI_QUERY_DEVICE_TYPE, &dev_type);
	errs	+= sdi_query(fd, -1, 0, SDI_QUERY_FGEN_MAX, &fgen_max);
	errs	+= sdi_query(fd, -1, 0, SDI_QUERY_FGEN_MIN, &fgen_min);
	errs	+= sdi_query(fd, -1, 0, SDI_QUERY_FSAMP_MAX, &fsamp_max);
	errs	+= sdi_query(fd, -1, 0, SDI_QUERY_FSAMP_MIN, &fsamp_min);
	errs	+= sdi_query(fd, -1, 0, SDI_QUERY_NDIV_MAX, &ndiv_max);
	errs	+= sdi_query(fd, -1, 0, SDI_QUERY_NDIV_MIN, &ndiv_min);
	errs	+= sdi_query(fd, -1, 0, SDI_QUERY_NRATE_MAX, &nrate_max);
	errs	+= sdi_query(fd, -1, 0, SDI_QUERY_NRATE_MIN, &nrate_min);

	fsamp_want	= (double) LIMIT_RANGE(fsamp, fsamp_min, fsamp_max);

	for (nrate_inc = nrate_min; nrate_inc <= nrate_max; nrate_inc++)
	{
		for (ndiv_inc = ndiv_min; ndiv_inc <= ndiv_max; ndiv_inc++)
		{
			switch (dev_type)
			{
				default:

					errs++;

				case GSC_DEV_TYPE_6SDI:
				case GSC_DEV_TYPE_16SDI:

					fgen_got	= SDI_FGEN_BASIC_KHZ(nrate_inc);
					fsamp_got	= SDI_FSAMP_BASIC_KSPS(nrate_inc, ndiv_inc);

					fgen_got	*= 1000;
					fsamp_got	*= 1000;
					break;

				case GSC_DEV_TYPE_16SDI_HS:
				case GSC_DEV_TYPE_16HSDI:

					fgen_got	= SDI_FGEN_HS_MHZ(nrate_inc);
					fsamp_got	= SDI_FSAMP_HS_MSPS(nrate_inc, ndiv_inc);

					fgen_got	*= 1000000;
					fsamp_got	*= 1000000;
					break;
			}

			if ((init) && ((fgen_got < fgen_min) || (fgen_got > fgen_max)))
				continue;

			if ((init) && ((fsamp_got < fsamp_min) || (fsamp_got > fsamp_max)))
				continue;

			delta	= fsamp_got - fsamp_want;

			if (ABS(delta) < ABS(delta_best))
			{
				delta_best	= delta;
				fsamp_best	= fsamp_got;
				nrate_best	= nrate_inc;
				ndiv_best	= ndiv_inc;
			}
		}
	}

	nrate[0]	= nrate_best;
	ndiv[0]		= ndiv_best;

	if (rate)
		rate[0]	= fsamp_best;

	if (verbose)
	{
		printf("%s  (", errs ? " FAIL <---" : "");
		printf("Nrate ");
		gsc_label_long_comma((long) nrate[0]);
		printf(", Ndiv ");
		gsc_label_long_comma((long) ndiv[0]);
		printf(", Fsamp %.3f", fsamp_best);
		printf(")\n");
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	sdi_fsamp_ai_report
*
*	Purpose:
*
*		Determine and report the sample rate for the specified channel.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		chan	The index of the channel of interest.
*
*		fsamp	The overall sample rate is reported here.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int sdi_fsamp_ai_report(int fd, int index, int verbose, int chan, s32* fsamp)
{
	char	buf[64];
	int		c_p_g;	// channels per group
	s32		chans;
	int		errs	= 0;
	int		group;
	s32		groups;
	s32		model;
	s32		nrate	= 0;
	s32		nrate_max;
	s32		ndiv;
	float	rate	= 0;
	s32		src;

	if (verbose)
	{
		sprintf(buf, "Channel %d", chan);
		gsc_label_index(buf, index);
	}

	for (;;)
	{
		if ((chan < 0) || (chan > 15))
		{
			errs	= 1;

			if (verbose)
				printf("FAIL <---  (invalid channel index: %d)\n", chan);

			break;
		}

		errs	+= sdi_query(fd, -1, 0, SDI_QUERY_DEVICE_TYPE, &model);
		errs	+= sdi_query(fd, -1, 0, SDI_QUERY_NRATE_MAX, &nrate_max);
		errs	+= sdi_query(fd, -1, 0, SDI_QUERY_CHANNEL_GPS, &groups);
		errs	+= sdi_query(fd, -1, 0, SDI_QUERY_CHANNEL_QTY, &chans);
		c_p_g	= chans / groups;
		group	= chan / c_p_g;
		errs	+= sdi_rate_div_ndiv_chan(fd, -1, 0, chan, -1, &ndiv);
		errs	+= sdi_ch_grp_src_group(fd, -1, 0, group, -1, &src);

		switch (src)
		{
			default:

				errs	= 1;

				if (verbose)
					printf("FAIL <---  (invalid source option: %ld)\n", (long) src);

				break;

			case SDI_CH_GRP_SRC_GEN_A:

				errs	+= sdi_rate_gen_a_nrate(fd, -1, 0, -1, &nrate);
				break;

			case SDI_CH_GRP_SRC_GEN_B:

				errs	+= sdi_rate_gen_b_nrate(fd, -1, 0, -1, &nrate);
				break;

			case SDI_CH_GRP_SRC_GEN_C:

				errs	+= sdi_rate_gen_c_nrate(fd, -1, 0, -1, &nrate);
				break;

			case SDI_CH_GRP_SRC_GEN_D:

				errs	+= sdi_rate_gen_d_nrate(fd, -1, 0, -1, &nrate);
				break;

			case SDI_CH_GRP_SRC_EXTERN:
			case SDI_CH_GRP_SRC_DISABLE:

				nrate	= 0;
				break;
		}

		if (errs)
		{
		}
		else if (src == SDI_CH_GRP_SRC_DISABLE)
		{
			rate	= 0;

			if (verbose)
				printf("Disabled\n");
		}
		else if (src == SDI_CH_GRP_SRC_EXTERN)
		{
			rate	= -1;

			if (verbose)
				printf("External Source\n");
		}
		else if (	(model == GSC_DEV_TYPE_6SDI)	||
					(model == GSC_DEV_TYPE_16SDI))
		{
			rate	= SDI_FSAMP_BASIC_KSPS(nrate, ndiv);
			rate	*= 1000;

			if (verbose)
			{
				gsc_label_long_comma(rate);
				printf(" S/S\n");
			}
		}
		else	// High Speed boards
		{
			rate	= SDI_FSAMP_HS_MSPS(nrate, ndiv);
			rate	*= 1000000;

			if (verbose)
			{
				gsc_label_long_comma(rate);
				printf(" S/S\n");
			}
		}

		break;
	}

	if (fsamp)
		fsamp[0]	= (s32) rate;

	return(errs);
}



/******************************************************************************
*
*	Function:	sdi_fsamp_ai_report_all
*
*	Purpose:
*
*		Determine and report the sample rate for all channels.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	The overall sample rate is reported here.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int sdi_fsamp_ai_report_all(int fd, int index, int verbose, s32* fsamp)
{
	int	chan;
	s32	chans	= 0;
	int	errs;
	s32	sps;
	s32	total	= 0;

	if (verbose)
		gsc_label_index("Sample Rates", index);

	errs	= sdi_query(fd, -1, 0, SDI_QUERY_CHANNEL_QTY, &chans);

	if (errs)
		chans	= 16;
	else if (verbose)
		printf("\n");

	gsc_label_level_inc();

	for (chan = 0; chan < chans; chan++)
	{
		errs	+= sdi_fsamp_ai_report(fd, -1, verbose, chan, &sps);
		total	+= sps;
	}

	if (verbose)
	{
		gsc_label("Overall Rate");
		gsc_label_long_comma(total);
		printf(" S/S\n");
	}

	gsc_label_level_dec();

	if (fsamp)
		fsamp[0]	= total;

	return(errs);
}


